#include <memory.h>
#include "html_wrapper.h"

/* This function is mandatory to process the html file. 
 * Basically a html file is a tree of tokens called nodes and
 * each node has properties and possible children

 * this is maybe the most difficult function to understand 
 * however you don't need to understand all the stuff just 
 * the inner -if- that will check if the element should be 
 * processed
 */
static void ProcessTree(xmlNode * node)
{
	xmlNode *cur_node = NULL;
	for (cur_node = node; cur_node; cur_node = cur_node->next) 
	{
		if(cur_node->type == XML_ELEMENT_NODE)  
		{
			// For this example, we want to retrieve all the URLs contained in the webpage
			if (strncmp((const char *)cur_node->name, "a", 1) == 0)
			{
				// All html urls are encoded like that <a href="url">Some Link</a>
				xmlChar* value = xmlGetProp(cur_node, (const xmlChar*)"href");
				char address[1000];
				// address will be the 'URL'
				memcpy(address, value, xmlStrlen(value)); 
				// xmlNodeGetContent(cur_node) will contain the 'Some Link'
				printf("URL found: %s --> %s\n", xmlNodeGetContent(cur_node), address);								
				memset(address, 0, 1000);
			}
			// We also want to retrieve H2 tokens (which are basically headers)
			else if(strncmp((const char *)cur_node->name, "h2", 2) == 0)
			{
				// Each H2 is encoded like that <h2>Title</h2>
				// xmlNodeGetContent(cur_node) will contain the 'Title'
				printf("H2  found: %s\n",xmlNodeGetContent(cur_node));
			}
		}
		// Since the html file is a tree we have to process each children of the current node
		ProcessTree(cur_node->children);
	}
}


// Again, this is only useful for the example .... not needed for the project
int LoadFileIntoBuffer(const char * name, char ** buffer)
{
	FILE * fd = fopen(name, "r");

	if(!fd)
	{
		#ifdef VERBOSE
		fprintf(stderr, "[ANALYZER] File not found\n");
		#endif 
		return -1;
	}

	fseek (fd , 0 , SEEK_END);
  	int size = ftell (fd);
  	rewind (fd);

  	*buffer = (char*) malloc(sizeof(char)*size);
  	if (!(*buffer)) 
  	{
  		#ifdef VERBOSE
  		fprintf(stderr, "[ANALYZER] Buffer alloc error\n");
  		#endif 

  		return -2;
  	}

	int result = fread(*buffer, 1, size, fd);
  	if (result != size)
  	{
  		#ifdef VERBOSE
  		fprintf(stderr, "[ANALYZER] Reading error\n");
  		#endif 

  		return -3;
  	}

  	fclose(fd);

  	return 1;
}

void usage(const char * exec)
{
	printf("----------------------------------------\n");
	printf("Usage: %s file1 (file2 ... filen)\n", exec);
	printf("----------------------------------------\n");
}

int main(int argc, char * argv[])
{
	if(argc < 2)
  	{
  		usage(argv[0]);
  		return 0;
  	}
  	
	char * buffer;	

	for(int i = 1; i < argc; i++)
	{
		HtmlParserPtr parser = InitHtmlParser();

		// This function is only required for the example
		// because the data is not in a buffer but stored in files
		if(LoadFileIntoBuffer(argv[i], &buffer) < 0)
	  	{
	    	#ifdef VERBOSE
	    	fprintf(stderr, "[ANALYZER] Loading error\n");
	    	#endif 
	    
	    	return -1;
	  	}

	  	if(SetBufferToProcess(parser, buffer))
	  	{
	    	#ifdef VERBOSE
	    	fprintf(stderr, "[ANALYZER] Processing error\n");
	    	#endif   		
	  	}

		#ifdef VERBOSE
		printf("[ANALYZER] Starting processing %s\n", argv[i]);
		#endif 	  	
		// Process the html page - explanations at the beginning of this file
	  	ProcessTree(xmlDocGetRootElement(parser->myDoc));
		#ifdef VERBOSE
		printf("[ANALYZER] Done processing %s\n", argv[i]);
		#endif   	

		CloseHtmlParser(parser);
	}
	
	return 1;
}
