#include "curl_wrapper.h"
#include <stdlib.h>
#include <memory.h>
#include <curl/curl.h>

typedef struct
{
	char * buffer;
	size_t size;
} MemoryChunk;

static CURL* curlHandle;
static MemoryChunk chunk;

static size_t write_curl_data(void * contents, size_t size, size_t nmemb, void * userp)
{
	size_t realsize = size * nmemb;
	MemoryChunk * chunk = (MemoryChunk *)userp;
	
	chunk->buffer = realloc(chunk->buffer, chunk->size + realsize + 1);
  	
	if (chunk->buffer == NULL)
  	{
		#ifdef DEBUG  		
    	fprintf(stderr, "[CURL] Writing Data Failed: Out of memory\n");
    	#endif

    	return 0;
  	}
	
  	memcpy(&(chunk->buffer[chunk->size]), contents, realsize);
  	chunk->size += realsize;
  	chunk->buffer[chunk->size]= 0;
	
  	return realsize;
}

int init_curl_wrapper()
{
	curl_global_init(CURL_GLOBAL_ALL);
	curlHandle = curl_easy_init();

	if(!curlHandle)
		return 0;

	CURLcode errorCode;
	if( (errorCode = curl_easy_setopt(curlHandle, CURLOPT_WRITEFUNCTION, write_curl_data)) != CURLE_OK)
	{
		#ifdef DEBUG		
		fprintf(stderr, "[CURL] Error while setting write function %s\n", curl_easy_strerror(errorCode));
		#endif

		return 0;
	}

	if( (errorCode = curl_easy_setopt(curlHandle, CURLOPT_WRITEDATA, (void *)&chunk)) != CURLE_OK)
	{
		#ifdef DEBUG		
		fprintf(stderr, "[CURL] Error while setting write buffer %s\n", curl_easy_strerror(errorCode));
		#endif
		
		return 0;
	}

	if( (errorCode = curl_easy_setopt(curlHandle, CURLOPT_USERAGENT, "projetIMA3_pa/1.0")) != CURLE_OK)
	{
		#ifdef DEBUG		
		fprintf(stderr, "[CURL] Error while setting user-agent %s\n", curl_easy_strerror(errorCode));
		#endif
		
		return 0;
	}

	if( (errorCode = curl_easy_setopt(curlHandle, CURLOPT_FOLLOWLOCATION, 1)) != CURLE_OK)
	{
		#ifdef DEBUG		
		fprintf(stderr, "[CURL] Error while enabling redirection following %s\n", curl_easy_strerror(errorCode));
		#endif
		
		return 0;
	}

	return 1;
}

int fetch_web_page(const char * address, char ** buffer, int * size)
{
	chunk.size = 0; // forcing reset
	
	curl_easy_setopt(curlHandle, CURLOPT_URL, address);
	CURLcode errorCode = curl_easy_perform(curlHandle);

	
	if(errorCode != CURLE_OK)
	{
		#ifdef DEBUG
		fprintf(stderr, "[CURL] Error while fetching page %s\n", curl_easy_strerror(errorCode));
		#endif
		
		*buffer = NULL;
		*size = 0;

		return 0;
	}

	*buffer = chunk.buffer;
	*size = chunk.size;

	return (errorCode == CURLE_OK);
}

int is_latest_url_text_html()
{
	long response;
	CURLcode errorCode;

	if((errorCode = curl_easy_getinfo(curlHandle , CURLINFO_CONTENT_TYPE , &response)) != CURLE_OK)
	{
		#ifdef DEBUG		
		fprintf(stderr, "[CURL] Error while getting info %s\n", curl_easy_strerror(errorCode));
		#endif
	}

	if(strncmp("text/html", (char *)response, 9) != 0)	
		return 0;
	return 1;
}

int get_effective_url_from_latest_url(char * effective_url)
{
	long response;
	CURLcode errorCode;

	if((errorCode = curl_easy_getinfo(curlHandle , CURLINFO_EFFECTIVE_URL , &response)) != CURLE_OK)
	{
		#ifdef DEBUG
		fprintf(stderr, "[CURL] Error while getting info %s\n", curl_easy_strerror(errorCode));
		#endif

		return 0;
	}

	strcpy(effective_url, (char*)response);	
	return 1;
}

int close_curl_wrapper()
{
	curl_easy_cleanup(curlHandle);	
	curl_global_cleanup();
	return 1;
}
